﻿using Proto;
using Proto.Extensions;

namespace TestProject;

/// <summary>
/// 프로그램
/// </summary>
class Program
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 프로그램 시작하기 - Main()

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    /// <returns></returns>
    private static async Task Main()
    {
        ActorSystem actorSystem = new ActorSystem
        (
            new ActorSystemConfig()
                .WithDeadLetterThrottleInterval(TimeSpan.FromSeconds(1))
                .WithDeadLetterThrottleCount(3)
        );

        actorSystem.EventStream.Subscribe<DeadLetterEvent>
        (
            deadLetter => Console.WriteLine
            (
                $"DeadLetter from {deadLetter.Sender} to {deadLetter.Pid} : {deadLetter.Message.GetMessageTypeName()} = '{deadLetter.Message}'"
            )
        );

        Props props = Props.FromFunc(context => Task.CompletedTask);

        PID pid = actorSystem.Root.Spawn(props);

        await actorSystem.Root.StopAsync(pid);

        for(int i = 0; i < 1000; i++)
        {
            actorSystem.EventStream.Publish(new DeadLetterEvent(pid, i, null));
        }

        await Task.Delay(TimeSpan.FromSeconds(2));

        for(int i = 0; i < 1000; i++)
        {
            actorSystem.EventStream.Publish(new DeadLetterEvent(pid, i, null));
        }
    }

    #endregion
}